/**
 * Netwosix Web Admin - libagent
 * Copyright (C) 2006 netwosix team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you want to contact e-mail at info@netwosix.org
 */

#include "agent.h"



/**
 */
AgentResponse * agent_response_new(void)
{
	AgentResponse * response = al_new(AgentResponse);

	response->next = response->prev = NULL;

	response->id = NULL;
	response->type = AGENT_RESPONSE_OK;
	response->agent_data = NULL;

	return response;
}


/**
 */
void agent_response_delete(AgentResponse * response)
{

	al_return_if_fail (response);

	al_delete(response->id);
	al_delete(response->agent_data);
	al_delete(response);

}


/**
 */
AgentResponsePacket * agent_response_packet_new(void)
{
	AgentResponsePacket * response_packet = al_new(AgentResponsePacket);

	response_packet->responses = NULL;

	return response_packet;
}


/**
 */
void agent_response_packet_delete(AgentResponsePacket * response_packet)
{

	al_return_if_fail (response_packet);

	if (response_packet->responses)
	{
		AgentResponse * response = response_packet->responses;
		AgentResponse * next;

		while (response)
		{
			next = response->next;
			agent_response_delete(response);
			response = next;
		}
	}
	
	al_delete(response_packet);

}


/**
 */
void agent_response_packet_response_add(AgentResponsePacket * response_packet, AgentResponse * response)
{

	al_return_if_fail (response_packet && response);

	if (response_packet->responses == NULL)
	{
		response_packet->responses = response;
		return;
	}


	response->next = response_packet->responses->next;
	response->prev = response_packet->responses;

	response_packet->responses->next = response;

}


/**
 */
AgentResponse * agent_response_from_request(const AgentRequest * request, agent_response_create_func func)
{
	AgentResponse * response;
	AlString      * data = NULL;



	al_return_if_fail (request && func);

	response = agent_response_new();
	agent_response_set_id(response, agent_request_get_id(request));

	if (al_xml_verify_from_string(agent_request_get_agent_data(request)) && agent_request_get_agent_data(request) != NULL)
	{
		al_log (("Invalid request agent-data\n"));
		agent_response_set_type(response, AGENT_RESPONSE_AGENT_INVALID_DATA);
	}
	else
	{
		al_log (("processing request id    : %s\n", agent_request_get_id(request)));
		al_log (("processing agent data    : %s\n", agent_request_get_agent_data(request)));

		data = (*func)(request, agent_request_get_agent_data(request));

		if (al_xml_verify_from_string(al_string_get(data)))
		{
			al_log (("Invalid xml response agent-data\n"));
			response->type = AGENT_RESPONSE_AGENT_INVALID_DATA;
		}
		else
		{
			agent_response_set_agent_data(response,al_string_get(data));
		}
	}


	al_string_delete(data);


	return response;
}


/**
 */
AgentResponsePacket * agent_response_packet_from_request_packet(const AgentRequestPacket * request_packet, agent_response_create_func func)
{
	AgentRequest        * request;
	AgentResponse       * response;
	AgentResponsePacket * response_packet;


	al_return_val_if_fail (request_packet, NULL);


	response_packet = agent_response_packet_new();

	request = request_packet->requests;
	while (request)
	{
		response = agent_response_from_request(request, func);
		if (response)
			agent_response_packet_response_add(response_packet, response);

		request = request->next;
	}

	return response_packet;
}


/**
 */
AlXmlList * agent_response_packet_to_xml(const AgentResponsePacket * response_packet)
{
	AlXmlList     * list = al_xml_list_new();
	AgentResponse * response;


	al_return_val_if_fail (response_packet, NULL);

	/*
	 * <agent>
	 *	<responses>
	 *		<response id=”00000000” type=”type”data=”data”/>
	 *	</responses>
	 * </agent>
	 */

	list->parent = al_xml_node_new();
	al_xml_node_set_name(list->parent, "agent");

	list->node = al_xml_node_new();
	al_xml_node_set_name(list->node, "responses");
	for (response = response_packet->responses;response;response = response->next)
	{

		al_xml_list_node_add(list, agent_response_to_xml(response));

	}

	return list;
}


/**
 */
AlXmlNode * agent_response_to_xml(const AgentResponse * response)
{
	AlXmlNode * node;
	char        stype[10];


	al_return_val_if_fail (response, NULL);

	node = al_xml_node_new();
	al_xml_node_set_name(node, "response");

	al_xml_node_attribute_new(node, "id", agent_response_get_id(response));

	if (agent_response_get_type(response) == AGENT_RESPONSE_OK)
	{
		// Return agent data only if it is ok
		al_xml_node_attribute_new(node, "data", agent_response_get_agent_data(response));
	}

	sprintf(stype,"%d",agent_response_get_type(response));
	al_xml_node_attribute_new(node, "type", stype);

	return node;
}


/**
 */
AlString * agent_response_packet_to_xml_string(const AgentResponsePacket * response_packet)
{
	AlXmlList * list;
	AlString  * s = NULL;


	al_return_val_if_fail(response_packet, NULL);

	list = agent_response_packet_to_xml(response_packet);
	if (list) s = al_xml_list_to_string(list, "../dtd/agent-response.dtd", "agent-response");


	return s;
}

